//
//  AppDelegate.m
//  DiskCopy2Dsk
//
//  Created by Chamberlin Family on 12/16/13.
//  Copyright (c) 2013 Stephen Chamberlin. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate
@synthesize outputDirectoryTextField;
@synthesize window = _window;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    outputDirectory = [paths objectAtIndex:0];
    [outputDirectoryTextField setStringValue:outputDirectory];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender
{
    return YES;
}

- (IBAction)handleConvert:(id)sender 
{
    NSOpenPanel* openDlg = [NSOpenPanel openPanel];
    [openDlg setCanChooseFiles:YES];
    [openDlg setAllowsMultipleSelection:YES];
    [openDlg setPrompt:@"Select"];
    [openDlg setMessage:@"Select DiskCopy 4.2 image files to convert"];
    
    if ([openDlg runModal] == NSOKButton)
    {
        NSArray* files = [openDlg filenames];
        
        for (int i=0; i<[files count]; i++)
        {
            // read the binary data from the file
            NSString* filename = [files objectAtIndex:i];
            NSData* fileData = [NSData dataWithContentsOfFile:filename];
            if (fileData == nil)
            {
                NSAlert* alert = [[NSAlert alloc] init];
                [alert addButtonWithTitle:@"OK"];
                [alert setMessageText:[NSString stringWithFormat: @"An error occurred while opening %@", [filename lastPathComponent]]];
                [alert setAlertStyle:NSCriticalAlertStyle];
                [alert runModal];
                return;
            }
            
            // ensure the file is at least 400K, and has the DiskCopy 4.2 magic bytes
            const uint8_t* bytes = [fileData bytes];
            if ([fileData length] < 400*1024 || bytes[0x52] != 0x01 || bytes[0x53] != 0x00)
            {
                NSAlert* alert = [[NSAlert alloc] init];
                [alert addButtonWithTitle:@"OK"];
                [alert setMessageText:[NSString stringWithFormat: @"%@ is not a DiskCopy 4.2 image file", [filename lastPathComponent]]];
                [alert setAlertStyle:NSCriticalAlertStyle];
                [alert runModal];
                return;
            }
            
            // ensure the embedded data size is 400, 800, or 1440 K
            uint32_t dataSize = bytes[0x40] * 16777216 + bytes[0x41] * 65536 + bytes[0x42] * 256 + bytes[0x43];
            if (dataSize != 400*1024 && dataSize != 800*1024 && dataSize != 1440*1024)
            {
                NSAlert* alert = [[NSAlert alloc] init];
                [alert addButtonWithTitle:@"OK"];
                [alert setMessageText:[NSString stringWithFormat: @"%@ is an unsupported image size", [filename lastPathComponent]]];
                [alert setAlertStyle:NSCriticalAlertStyle];
                [alert runModal];
                return;
            }
            
            // get the embedded data, and write it out as a new file with a .dsk extension
            NSData* imageData = [fileData subdataWithRange:NSMakeRange(0x54, dataSize)];
            NSString* saveFilename = [NSString stringWithFormat:@"%@/%@.dsk", outputDirectory, [[filename lastPathComponent] stringByDeletingPathExtension]];
            if (![imageData writeToFile:saveFilename atomically:NO])
            {
                NSAlert* alert = [[NSAlert alloc] init];
                [alert addButtonWithTitle:@"OK"];
                [alert setMessageText:[NSString stringWithFormat: @"An error occurred while saving %@", [saveFilename lastPathComponent]]];
                [alert setAlertStyle:NSCriticalAlertStyle];
                [alert runModal];
                return;
            }
        }
        
        NSAlert* alert = [[NSAlert alloc] init];
        [alert addButtonWithTitle:@"OK"];
        [alert setMessageText:@"Success!"];
        [alert setAlertStyle:NSInformationalAlertStyle];
        [alert runModal];
    }
    
}
- (IBAction)handleChangeOuputDirectory:(id)sender 
{
    NSOpenPanel* openDlg = [NSOpenPanel openPanel];
    [openDlg setCanChooseFiles:NO];
    [openDlg setCanChooseDirectories:YES];
    [openDlg setAllowsMultipleSelection:NO];
    [openDlg setPrompt:@"Select"];
    [openDlg setMessage:@"Select a Directory"];
    
    if ([openDlg runModal] == NSOKButton)
    {
        NSArray* files = [openDlg filenames];
        outputDirectory = [files objectAtIndex:0];
        [outputDirectoryTextField setStringValue:outputDirectory];
    }
}
@end
